@@ -9,11 +9,11 @@ import os
 
 from pybi.core.components import (
     ContainerComponent,
-    ColBoxComponent,
     ComponentTag,
     BoxComponent,
+    ForeachBoxComponent,
     FlowBoxComponent,
-    TextComponent,
+    SizebarComponent,
     UploadComponent,
     GridBoxComponent,
     TabsComponent,
@@ -24,6 +24,8 @@ from pybi.core.components import (
     Mermaid,
     Input,
     NumberSlider,
+    SpaceComponent,
+    Checkbox,
 )
 from pybi.core.components.reactiveComponent import EChart, Slicer, Table, TextValue
 from pybi.core.dataSource import (
@@ -43,7 +45,7 @@ from pybi.utils.data_gen import (
 from pybi.utils.markdown2 import markdown
 from pybi.core.uiResource import ResourceManager
 
-from pybi.core.sql import SqlInfo, SqlWrapper
+from pybi.core.sql import SqlInfo, SqlWrapper, ForeachRowInfo, extract_sql_text
 import pybi.utils.sql as sqlUtils
 from pybi.easyEcharts.base import BaseChart, ChartCollector
 from pybi.easyEcharts import easy_echarts
@@ -56,10 +58,13 @@ from pybi.core.components.reactiveComponent.echarts import (
 import pybi.utils.pyecharts_utils as pyecharts_utils
 from pybi.icons.iconManager import get_singleton as get_iconManager
 from pybi.core.webResources import WebResourceManager
+from pybi.core.styles.styleTag import StyleTagInfo
 import pybi as pbi
 
 if TYPE_CHECKING:
     pass
+    # from pybi.core.sql import ForeachRowInfo
+
     # from pybi.core.components import ReactiveComponent
 
 
@@ -86,6 +91,12 @@ class AppMeta:
         self.__app._doc_title = title
         return self
 
+    def add_style_tag(
+        self, css: str, id: Optional[str] = None, media: Optional[str] = None
+    ):
+        self.__app._style_tags.append(StyleTagInfo(css, id, media))
+        return self
+
 
 class App(ContainerComponent):
     def __init__(self) -> None:
@@ -97,7 +108,7 @@ class App(ContainerComponent):
         self._clear_data = False
         self.dbLocalStorage = False
         self.echartsRenderer = "canvas"
-        self._doc_title = None
+        self._doc_title: Optional[str] = None
         self.__meta = AppMeta(self)
         self.__json_utils = JsonUtils()
         self.__resourceManager = ResourceManager()
@@ -105,6 +116,9 @@ class App(ContainerComponent):
 
         self.__webResourceManager = WebResourceManager()
         self.webResources = []
+        self._style_tags: List[StyleTagInfo] = []
+
+        self.__has_sizebar = False
 
     def __record_and_check_dataset_name(self, name: str):
         if name in self.__dataSetNames:
@@ -192,13 +206,14 @@ class App(ContainerComponent):
 
     def add_text(
         self,
-        text: Union[str, SqlWrapper],
+        text: Union[str, SqlWrapper, ForeachRowInfo],
         *,
         host: Optional[ContainerComponent] = None,
     ):
-        if isinstance(text, SqlWrapper):
+        if isinstance(text, (SqlWrapper, ForeachRowInfo)):
             text = str(text)
-        contexts = list(SqlInfo.extract_sql_from_text(text))
+
+        contexts = list(extract_sql_text(text))
 
         cp = TextValue(contexts)
 
@@ -219,7 +234,7 @@ class App(ContainerComponent):
             Defaults `None`:  select distinct colA from data
             '1' : select distinct colA from data order by 1
             '1 desc' : select distinct colA from data order by 1 desc
-            'colX dsce' : select distinct colA from data order by colX desc
+            'colX desc' : select distinct colA from data order by colX desc
         """
         if isinstance(field, DataSourceTable):
             field = field[field.columns[0]]
@@ -237,6 +252,36 @@ class App(ContainerComponent):
         self.__resourceManager.register_element_cps()
         return cp
 
+    def add_checkbox(
+        self,
+        field: Union[DataSourceField, DataSourceTable],
+        *,
+        orderby: Optional[str] = "1",
+        host: Optional[ContainerComponent] = None,
+    ):
+        """
+        orderby:
+            Defaults '1' : select distinct colA from data order by 1
+            `None`:  select distinct colA from data
+            '1 desc' : select distinct colA from data order by 1 desc
+            'colX desc' : select distinct colA from data order by colX desc
+        """
+        if isinstance(field, DataSourceTable):
+            field = field[field.columns[0]]
+
+        assert isinstance(field, DataSourceField)
+        orderSql = "" if orderby is None else f"order by {orderby}"
+        sql = f"select distinct {field._get_sql_field_name()} from {field.source_name} {orderSql}"
+        cp = Checkbox(SqlInfo(sql))
+        cp.title = field.name
+        cp.add_updateInfo(field.source_name, field._get_sql_field_name())
+
+        host = host or self._get_temp_host() or self
+        host._add_children(cp)
+
+        # self.__resourceManager.register_element_cps()
+        return cp
+
     def add_table(
         self,
         dataSourceTable: DataSourceTable,
@@ -306,7 +351,9 @@ class App(ContainerComponent):
                 for map_name in mapIds:
                     self.__webResourceManager.mark_echarts_map(map_name)
 
-                info = EChartInfo(opts, ds_infos, updateInfos, jscodes, mapIds)
+                info = EChartInfo(
+                    opts, ds_infos, updateInfos, jscodes, mapIds, chart._merge_opt
+                )
                 cp._add_chart_info(info)
 
         elif isinstance(options, dict):
@@ -336,10 +383,35 @@ class App(ContainerComponent):
 
     def flowBox(
         self,
+        align: Optional[str] = None,
         *,
         host: Optional[ContainerComponent] = None,
     ):
-        cp = FlowBoxComponent(self)
+        """行容器，里面的组件会横向放置，在超过屏幕宽度时，自动换行
+
+        Args:
+            align (Optional[str], optional): 容器中的元素组件靠向哪个位置。'left' | 'center' | 'right' | 'between' | 'around' |'evenly' |'stretch'. Defaults to None('left').
+
+                - left : 从行首起始位置开始排列
+                - center:居中排列
+                - right:从行尾位置开始排列
+                - between: 均匀排列每个元素,首个元素放置于起点，末尾元素放置于终点
+                - around:  均匀排列每个元素,每个元素周围分配相同的空间
+                - evenly:  均匀排列每个元素, 每个元素之间的间隔相等
+
+        ## 示例
+
+        >>> # 2个文本放置在同一行，并且靠左放置
+        >>> with pbi.flowBox():
+                pbi.add_text('text A')
+                pbi.add_text('text B')
+
+        >>> # 2个文本放置在同一行，并且居中放置
+        >>> with pbi.flowBox('center'):
+                pbi.add_text('text A')
+                pbi.add_text('text B')
+        """
+        cp = FlowBoxComponent(align=align, appHost=self)
 
         host = host or self._get_temp_host() or self
         host._add_children(cp)
@@ -349,13 +421,55 @@ class App(ContainerComponent):
     def gridBox(
         self,
         areas: Union[List[List[str]], str],
+        align: Optional[str] = None,
+        vertical_align: Optional[str] = None,
         *,
         host: Optional[ContainerComponent] = None,
     ):
+        """网格布局
+
+        Args:
+            areas (Union[List[List[str]], str]): 网格布局文本
+        ---
+        ### 示例
+        ```python
+        grid='''
+            a b c
+            a e .
+        '''
+
+        with pbi.gridBox(grid):
+            pbi.add_slicer(...).set_gridArea('a')
+            pbi.add_slicer(...).set_gridArea('b')
+        ```
+
+
+        当 `areas` 设置为 '' 时,默认为动态布局，每行放置3格
+        ```python
+        with pbi.gridBox(''):
+            # 3个切片器在第1行
+            pbi.add_slicer(...)
+            pbi.add_slicer(...)
+            pbi.add_slicer(...)
+
+            # 2个切片器在第2行
+            pbi.add_slicer(...)
+            pbi.add_slicer(...)
+        ```
+
+        可以通过 `auto_fill_fixed_num` 修改动态配置
+
+        >>> with pbi.gridBox('').auto_fill_fixed_num(2):
+                ...
+
+        """
         if isinstance(areas, str):
             areas = GridBoxComponent.areas_str2array(areas)
 
-        cp = GridBoxComponent(areas, self)
+        cp = GridBoxComponent(areas, align, vertical_align, self)
+
+        if len(areas) == 0:
+            cp.auto_fill_by_num()
 
         host = host or self._get_temp_host() or self
         host._add_children(cp)
@@ -368,23 +482,89 @@ class App(ContainerComponent):
         *,
         host: Optional[ContainerComponent] = None,
     ):
-        cp = ColBoxComponent(spec, self)
-
-        host = host or self._get_temp_host() or self
-        host._add_children(cp)
+        spec = spec or [1, 1]
+        grid_text = " ".join(f"x{n}" for n in range(len(spec)))
+        cp = self.gridBox(grid_text).set_columns_sizes(spec)  # type: ignore
 
         return cp
 
     def box(
         self,
+        align: Optional[str] = None,
+        vertical_align: Optional[str] = None,
         *,
         host: Optional[ContainerComponent] = None,
     ):
-        cp = BoxComponent(self)
+        """盒容器，里面的组件竖向放置
+
+        Args:
+            align (Optional[str], optional): 容器中的元素组件横向靠哪个位置。
+                'left' | 'center' | 'right' | 'between' | 'around' |'evenly'|'stretch'. Defaults to None('left').
+
+                - left : 从行首起始位置开始排列
+                - center:居中排列
+                - right:从行尾位置开始排列
+
+            vertical_align (Optional[str], optional): 容器中的元素垂直方向靠哪个位置。
+                'top' | 'center' | 'bottom' | 'between' | 'around' . Defaults to None('top').
+
+                - top : 靠顶部
+                - center:居中
+                - bottom:靠底部
+                - between:项目在行与行之间留有间隔
+                - around:项目在行之前、行之间和行之后留有空间
+
+        ## 示例
+
+        """
+        cp = BoxComponent(align=align, vertical_align=vertical_align, appHost=self)
         host = host or self._get_temp_host() or self
         host._add_children(cp)
         return cp
 
+    def foreach(
+        self,
+        sql: str,
+        *,
+        host: Optional[ContainerComponent] = None,
+    ):
+        """ """
+
+        cp = ForeachBoxComponent(sql=pbi.sql(sql)._sql_info, appHost=self)
+        host = host or self._get_temp_host() or self
+        host._add_children(cp)
+        return cp
+
+    def sizebar(
+        self,
+        *,
+        host: Optional[ContainerComponent] = None,
+    ):
+        """侧边栏
+
+        使用 element-plus 的 Drawer 组件，具体属性可以参考文档:
+        [点击打开文档](https://element-plus.org/zh-CN/component/drawer.html#drawer-attributes)
+
+
+        ## 示例
+
+        ```python
+        with pbi.sizebar():
+            pbi.add_slicer(data["name"])
+        ```
+
+        """
+
+        if self.__has_sizebar:
+            raise Exception("Cannot create multiple sizebar components")
+
+        cp = SizebarComponent(appHost=self)
+        host = host or self._get_temp_host() or self
+        host._add_children(cp)
+        self.__resourceManager.register_element_cps()
+        self.__has_sizebar = True
+        return cp
+
     def add_tabs(
         self,
         names: List[str],
@@ -420,34 +600,6 @@ class App(ContainerComponent):
             }
         )
 
-        # self.webResources.append(
-        #     {
-        #         "id": "maps1",
-        #         "type": "echarts-map",
-        #         "input": ds2sqlite_file_base64(
-        #             self.dataSources, clear_data=self._clear_data
-        #         ),
-        #         "actionPipe": [],
-        #     }
-        # )
-
-        # self.webResources.append(
-        #     {
-        #         "id": "maps1",
-        #         "type": "echarts-map",
-        #         "input": None,
-        #         "actionPipe": [
-        #             {
-        #                 "name": "fetch",
-        #                 "args": {
-        #                     "url": "https://geo.datav.aliyun.com/areas_v3/bound/100000_full.json",
-        #                     "options": {},
-        #                 },
-        #             }
-        #         ],
-        #     }
-        # )
-
         wrs = self.__webResourceManager.create_webResources()
         self.webResources.extend(wrs)
 
@@ -456,6 +608,9 @@ class App(ContainerComponent):
         if self._doc_title:
             data["docTitle"] = self._doc_title
 
+        if self._style_tags:
+            data["styleTags"] = self._style_tags
+
         data["version"] = pbi.__version__
         return data
 
@@ -487,13 +642,13 @@ class App(ContainerComponent):
 
     def to_html(self, file, display_output_path=False):
         try:
-            # file = Path(file)
+            file = Path(file)
             # raw = self.to_raw_html()
 
             raw = self.__resourceManager.build_html(
                 self.to_json(), svg_infos=get_iconManager().get_infos()
             )
-            Path(file).write_text(raw, "utf8")
+            file.write_text(raw, "utf8")
 
             if display_output_path:
                 print(f"to html:{file.absolute()}")
@@ -578,6 +733,7 @@ class App(ContainerComponent):
     def add_mermaid(
         self,
         graph: str,
+        name="main",
         *,
         host: Optional[ContainerComponent] = None,
     ):
@@ -593,7 +749,7 @@ class App(ContainerComponent):
         '''
         >>> add_mermaid(graph)
         """
-        cp = Mermaid(graph)
+        cp = Mermaid(graph, name)
         host = host or self._get_temp_host() or self
         host._add_children(cp)
 
@@ -602,14 +758,18 @@ class App(ContainerComponent):
 
     def add_input(
         self,
-        data: DataSourceTable,
-        field: str,
+        field: Union[DataSourceField, DataSourceTable],
         where_expr="like '%${}%'",
         *,
         host: Optional[ContainerComponent] = None,
     ):
         """ """
-        cp = Input(where_expr).add_updateInfo(data.source_name, field)
+        if isinstance(field, DataSourceTable):
+            field = field[field.columns[0]]
+
+        assert isinstance(field, DataSourceField)
+
+        cp = Input(where_expr).add_updateInfo(field.source_name, field.name)
         host = host or self._get_temp_host() or self
         host._add_children(cp)
 
@@ -636,3 +796,26 @@ class App(ContainerComponent):
 
         self.__resourceManager.register_element_cps()
         return cp
+
+    def space(
+        self,
+        space: Optional[str] = None,
+        *,
+        auto_fill_row=False,
+        host: Optional[ContainerComponent] = None,
+    ):
+        """增加组件之间的空间
+
+        Args:
+            space (Optional[str], optional): 间距值(默认为'0.2em'). Defaults to None.
+            auto_fill_row (bool, optional): 放在 flowBox 中,会迫使后续的组件往右靠. Defaults to False.
+        ---
+        >>> pbi.space()
+        >>> pbi.space('100px')
+        >>> pbi.space('0.5em')
+        """
+        cp = SpaceComponent(space, auto_fill_row)
+        host = host or self._get_temp_host() or self
+        host._add_children(cp)
+
+        return cp
