@@ -4,6 +4,7 @@ from pybi.core.components.staticComponent import (
     UploadComponent,
     IconComponent,
     SvgIconComponent,
+    SpaceComponent,
 )
 from pybi.core.components.reactiveComponent import (
     ReactiveComponent,
@@ -13,15 +14,17 @@ from pybi.core.components.reactiveComponent import (
     Markdown,
     Input,
     NumberSlider,
+    Checkbox,
 )
 from pybi.core.components.containerComponent import (
     ContainerComponent,
     BoxComponent,
-    ColBoxComponent,
     FlowBoxComponent,
     GridBoxComponent,
     TabsComponent,
     AffixComponent,
+    ForeachBoxComponent,
+    SizebarComponent,
 )
 
 from pybi.core.components.mermaid import Mermaid
