@@ -5,6 +5,7 @@ class ComponentTag(Enum):
     App = "App"
     DataSource = "DataSource"
     Slicer = "elSlicer"
+    Checkbox = "Checkbox"
     Input = "elInput"
     NumberSlider = "elNumberSlider"
     Table = "elTable"
@@ -22,3 +23,6 @@ class ComponentTag(Enum):
     SvgIcon = "SvgIcon"
     Affix = "Affix"
     Mermaid = "Mermaid"
+    Space = "Space"
+    Foreach = "Foreach"
+    Sizebar = "elSizebar"
