@@ -1,25 +1,37 @@
 from __future__ import annotations
 from typing import TYPE_CHECKING, Dict, Union, List, Optional
 
+from pybi.core.sql import ForeachRowInfo
+
 from .componentTag import ComponentTag
 from .component import Component
 from pybi.icons.iconManager import get_singleton as get_iconManager
 
 if TYPE_CHECKING:
     from pybi.app import App
+    from pybi.core.sql import SqlInfo
 
 
 class ContainerComponent(Component):
-    def __init__(self, tag: ComponentTag, appHost: Optional[App] = None) -> None:
+    def __init__(
+        self,
+        tag: ComponentTag,
+        appHost: Optional[App] = None,
+        childredHook: Optional[ContainerComponent] = None,
+    ) -> None:
         super().__init__(tag)
 
         # TODO: maybe use weak ref?
         self._appHost = appHost
 
+        self._childredHook = childredHook
         self.children: List[Component] = []
 
     def _add_children(self, stat: Component):
-        self.children.append(stat)
+        if self._childredHook:
+            self._childredHook._add_children(stat)
+        else:
+            self.children.append(stat)
         return self
 
     def __enter__(self):
@@ -33,24 +45,110 @@ class ContainerComponent(Component):
 
 
 class BoxComponent(ContainerComponent):
-    def __init__(self, appHost: Optional[App] = None) -> None:
+    m_align_mapping = {
+        "left": "flex-start",
+        "center": "center",
+        "right": "flex-end",
+        "stretch": "stretch",
+    }
+
+    m_vertical_align_mapping = {
+        "top": "flex-start",
+        "center": "center",
+        "bottom": "flex-end",
+        "between": "space-between",
+        "around": "space-around",
+    }
+
+    def __init__(
+        self,
+        align: Optional[str] = None,
+        vertical_align: Optional[str] = None,
+        appHost: Optional[App] = None,
+    ) -> None:
+        """
+        Args:
+            align (Optional[str], optional): 'left' | 'center' | 'right'. Defaults to None.
+            vertical_align (Optional[str], optional): 'top' | 'center' | 'bottom'. Defaults to None.
+        """
         super().__init__(ComponentTag.Box, appHost)
+        self._align = align
+        self._vertical_align = vertical_align
+
+    def _to_json_dict(self):
+        data = super()._to_json_dict()
+        if self._align:
+            data["align"] = self.m_align_mapping[self._align]
+
+        if self._vertical_align:
+            data["verticalAlign"] = self.m_vertical_align_mapping[self._vertical_align]
+
+        return data
 
 
 class FlowBoxComponent(ContainerComponent):
-    def __init__(self, appHost: Optional[App] = None) -> None:
+    m_align_mapping = {
+        "left": "flex-start",
+        "center": "center",
+        "right": "flex-end",
+        "between": "space-between",
+        "around": "space-around",
+        "evenly": "space-evenly",
+        "stretch": "stretch",
+    }
+
+    def __init__(
+        self, align: Optional[str] = None, appHost: Optional[App] = None
+    ) -> None:
+        """
+        Args:
+            align (Optional[str], optional): 'left' | 'center' | 'right'. Defaults to None.
+        """
         super().__init__(ComponentTag.FlowBox, appHost)
+        self._align = align
 
     def __get_item__(self, idx: int):
         return self.children[idx]
 
+    def _to_json_dict(self):
+        data = super()._to_json_dict()
+        if self._align:
+            data["align"] = self.m_align_mapping[self._align]
+
+        return data
+
 
 class GridBoxComponent(ContainerComponent):
-    def __init__(self, areas: List[List[str]], appHost: Optional[App] = None) -> None:
+    m_align_mapping = {
+        "left": "flex-start",
+        "center": "center",
+        "right": "flex-end",
+        "stretch": "stretch",
+    }
+
+    m_vertical_align_mapping = {
+        "top": "flex-start",
+        "center": "center",
+        "bottom": "flex-end",
+        "between": "space-between",
+        "around": "space-around",
+    }
+
+    def __init__(
+        self,
+        areas: Optional[List[List[str]]] = None,
+        align: Optional[str] = None,
+        vertical_align: Optional[str] = None,
+        appHost: Optional[App] = None,
+    ) -> None:
         super().__init__(ComponentTag.GridBox, appHost)
         self.__areas = areas
         self.__columns_sizes: List[str] = []
         self.__rows_sizes: List[str] = []
+        self.__gridTemplateColumns = None
+
+        self._align = align
+        self._vertical_align = vertical_align
 
     def set_columns_sizes(self, sizes: List[Union[str, int]]):
         """
@@ -73,38 +171,74 @@ class GridBoxComponent(ContainerComponent):
     def __get_item__(self, idx: int):
         return self.children[idx]
 
+    def auto_fill_by_width(self, width="15em"):
+        """动态调整每行组件数量，每个组件固定宽度为指定 `width`。
+        当设置有效 grid areas 时，此设置无效
+
+        Args:
+            width (str, optional): 每个组件固定宽度. Defaults to "15em".
+
+        """
+
+        width_value = f"minmax(0, {width})"
+        self.__gridTemplateColumns = f"repeat(auto-fill, {width_value})"
+        self.set_style("justify-content: space-between")
+        return self
+
+    def auto_fill_by_num(self, num=3):
+        """自动调整每行组件宽度，每行组件数量为 `num`。
+        当设置有效 grid areas 时，此设置无效
+
+        Args:
+            num (int, optional): 每行组件数量. Defaults to 3.
+
+        """
+        self.__gridTemplateColumns = f"repeat({num}, 1fr)"
+        return self
+
     def _to_json_dict(self):
         data = super()._to_json_dict()
-        data["areas"] = GridBoxComponent.areas_array2str(self.__areas)
 
-        areas_cols_len = max(map(len, self.__areas))
-        cols_size = GridBoxComponent.padded_grid_template(
-            self.__columns_sizes, areas_cols_len
-        )
+        if self.__areas:
+            data["areas"] = GridBoxComponent.areas_array2str(self.__areas)
 
-        areas_rows_len = len(self.__areas)
-        rows_size = GridBoxComponent.padded_grid_template(
-            self.__rows_sizes, areas_rows_len
-        )
+            areas_cols_len = max(map(len, self.__areas))
+            cols_size = GridBoxComponent.padded_grid_template(
+                self.__columns_sizes, areas_cols_len, "1fr"
+            )
 
-        data["gridTemplateColumns"] = " ".join(cols_size)
-        data["gridTemplateRows"] = " ".join(rows_size)
+            areas_rows_len = len(self.__areas)
+            rows_size = GridBoxComponent.padded_grid_template(
+                self.__rows_sizes, areas_rows_len
+            )
+
+            data["gridTemplateColumns"] = " ".join(cols_size)
+            data["gridTemplateRows"] = " ".join(rows_size)
+        else:
+            if self.__gridTemplateColumns:
+                data["gridTemplateColumns"] = self.__gridTemplateColumns
+
+        if self._align:
+            data["align"] = self.m_align_mapping[self._align]
+
+        if self._vertical_align:
+            data["verticalAlign"] = self.m_vertical_align_mapping[self._vertical_align]
 
         return data
 
     @staticmethod
-    def padded_grid_template(sizes: List[str], real_size: int):
+    def padded_grid_template(sizes: List[str], real_size: int, pad="auto"):
         """
-        >>> sizes = ['1rf']
+        >>> sizes = ['1fr']
         >>> real_size = 3
         >>> padded_grid_template(sizes,real_size)
-        >>> ['1rf','auto','auto']
+        >>> ['1fr','auto','auto']
         """
         diff_len = real_size - len(sizes)
         sizes = sizes.copy()
 
         if diff_len > 0:
-            sizes.extend(["auto"] * diff_len)
+            sizes.extend([pad] * diff_len)
         return sizes
 
     @staticmethod
@@ -151,23 +285,6 @@ class GridBoxComponent(ContainerComponent):
         return list(splie_space)
 
 
-class ColBoxComponent(ContainerComponent):
-    def __init__(
-        self, spec: Optional[List[int]] = None, appHost: Optional[App] = None
-    ) -> None:
-        super().__init__(ComponentTag.ColBox, appHost)
-
-        if spec is None:
-            spec = [1, 1]
-
-        self.spec = spec
-
-        assert self._appHost is not None, "self._appHost must be app instance"
-
-    def __getitem__(self, idx: int):
-        return self.children[idx]
-
-
 class TabsComponent(ContainerComponent):
     def __init__(self, names: List[str], appHost: Optional[App] = None) -> None:
         """
@@ -186,7 +303,7 @@ class TabsComponent(ContainerComponent):
         self.__name2idx = {name: idx for idx, name in enumerate(self.names)}
 
         for _ in range(len(self.names)):
-            self._add_children(BoxComponent(appHost))
+            self._add_children(BoxComponent(appHost=appHost))
 
     def __getitem__(self, idx: Union[int, str]):
         if isinstance(idx, str):
@@ -249,3 +366,31 @@ class TabsComponent(ContainerComponent):
 class AffixComponent(ContainerComponent):
     def __init__(self, appHost: Optional[App] = None) -> None:
         super().__init__(ComponentTag.Affix, appHost)
+        box = BoxComponent(appHost=appHost)
+        self._add_children(box)
+        self._childredHook = box
+
+
+class DynamicBoxComponent:
+    def __init__(self, grid_box: GridBoxComponent) -> None:
+        self.__grid_box = grid_box
+
+    def set_each_row_num(self, num: int):
+        pass
+
+    def set_each_width(self, width: str):
+        pass
+
+
+class ForeachBoxComponent(ContainerComponent):
+    def __init__(self, sql: SqlInfo, appHost: Optional[App] = None) -> None:
+        super().__init__(ComponentTag.Foreach, appHost)
+        self.sql = sql
+
+    def __getitem__(self, field: str):
+        return ForeachRowInfo(field)
+
+
+class SizebarComponent(ContainerComponent):
+    def __init__(self, appHost: Optional[App] = None) -> None:
+        super().__init__(ComponentTag.Sizebar, appHost)
