@@ -5,7 +5,7 @@ from pybi.utils.data_gen import Jsonable
 
 from pybi.core.components import ComponentTag
 from pybi.core.components.component import Component
-from pybi.core.sql import Sql
+from pybi.core.sql import SqlInfo
 
 from pybi.core.dataSource import DataSourceView, DataSourceTable
 import re
@@ -39,15 +39,10 @@ class ReactiveComponent(Component):
 
 
 class SingleReactiveComponent(ReactiveComponent):
-    def __init__(self, tag: ComponentTag, sql: Sql) -> None:
+    def __init__(self, tag: ComponentTag, sql: SqlInfo) -> None:
         super().__init__(tag)
-        self._sql = sql
+        self.sqlInfo = sql
 
-    def sql(self, sql: str):
-        self._sql = sql
-        return self
-
-    def _to_json_dict(self):
-        data = super()._to_json_dict()
-        data["sqlInfo"] = self._sql
-        return data
+    # def sql(self, sql: str):
+    #     self._sql = sql
+    #     return self
