@@ -120,12 +120,14 @@ class EChartInfo(Jsonable):
         updateInfos: List[EChartUpdateInfo],
         jscodes: List[EChartJscode],
         mapIds: Optional[List[str]] = None,
+        postMergeSettings: Optional[Dict] = None,
     ):
         self.options = options
         self.datasetInfos = datasetInfos
         self.updateInfos = updateInfos
         self.jsCodes = jscodes
         self.mapIds = mapIds or []
+        self.postMergeSettings = postMergeSettings or {}
 
 
 class EChart(ReactiveComponent):
