@@ -3,22 +3,22 @@ from typing import TYPE_CHECKING, List, Union
 
 
 from pybi.core.components import ComponentTag
-from pybi.core.sql import Sql
 from .base import SingleReactiveComponent
 
 
 if TYPE_CHECKING:
-    pass
+    from pybi.core.sql import SqlInfo
 
 
 class Table(SingleReactiveComponent):
     def __init__(
         self,
-        sql: Sql,
+        sql: SqlInfo,
     ) -> None:
         super().__init__(ComponentTag.Table, sql)
         self.pageSize = 10
         self.tableHeight = "initial"
+        self.tableWidth = "initial"
 
     def set_page_size(self, size: int):
         """
@@ -36,3 +36,11 @@ class Table(SingleReactiveComponent):
         """
         self.tableHeight = height
         return self
+
+    def set_table_width(self, width="initial"):
+        """
+        表格高度
+        width: 'initial'(默认值),'30em','30%','30vh'
+        """
+        self.tableWidth = width
+        return self
