@@ -7,10 +7,10 @@ from .base import ReactiveComponent
 
 
 if TYPE_CHECKING:
-    from pybi.core.sql import SqlInfo
+    from pybi.core.sql import SqlInfo, ForeachRowInfo
 
 
 class TextValue(ReactiveComponent):
-    def __init__(self, contexts: List[Union[str, SqlInfo]]) -> None:
+    def __init__(self, contexts: List[Union[str, SqlInfo, ForeachRowInfo]]) -> None:
         super().__init__(ComponentTag.TextValue)
         self.contexts = contexts
