@@ -1,7 +1,6 @@
 from __future__ import annotations
-import pybi.utils.sql as sqlUtils
 from pybi.utils.data_gen import Jsonable
-from typing import Dict, Generator, Optional, List
+from typing import Optional, List
 
 import re
 
@@ -9,6 +8,8 @@ m_sql_from_text_pat = re.compile(
     r"(?:info:\[\|sql:)(.+?)\|type:(.+?)\|js_map:(.*?)(?:\|])", re.I | re.DOTALL
 )
 
+m_foreach_text_pat = re.compile(r"(?:foreach:\[\|field:)(.+?)(?:\|])", re.I | re.DOTALL)
+
 
 class SqlWrapper(Jsonable):
     def __init__(self, sql: str) -> None:
@@ -81,6 +82,39 @@ if (fields.length>1){{
         return self._sql_info
 
 
+class ForeachRowInfo(Jsonable):
+    def __init__(self, field: str) -> None:
+        super().__init__()
+        self.field = field
+
+    @staticmethod
+    def extract_from_text(text: str):
+        """
+        >>> input = '字段名字:foreach:[|field:name|]'
+        >>> extract_from_text(input)
+        >>> ['字段名字:',ForeachRowInfo('name')]
+        """
+        start_idx = 0
+
+        for match in re.finditer(m_foreach_text_pat, text):
+            span = match.span()
+
+            if span[0] > start_idx:
+                # 前面有普通文本
+                yield text[start_idx : span[0]]
+
+            yield ForeachRowInfo(match.group(1))
+            start_idx = span[1]
+
+        end_idx = len(text) - 1
+
+        if start_idx <= end_idx:
+            yield text[start_idx : len(text)]
+
+    def __str__(self) -> str:
+        return f"foreach:[|field:{self.field}|]"
+
+
 class SqlInfo(Jsonable):
     def __init__(
         self, sql: str, type: str = "infer", js_map: Optional[str] = None
@@ -112,7 +146,6 @@ class SqlInfo(Jsonable):
         start_idx = 0
 
         for match in re.finditer(m_sql_from_text_pat, text):
-
             span = match.span()
 
             if span[0] > start_idx:
@@ -138,41 +171,11 @@ class SqlInfo(Jsonable):
         return m_sql_from_text_pat.sub(add_backtick, text)
 
 
-# class SqlExtractor:
-#     js_code_flag = "--x_x--0_0--"
+def extract_sql_text(sql_text: str):
+    for t in SqlInfo.extract_sql_from_text(sql_text):
+        if isinstance(t, str):
+            for t in ForeachRowInfo.extract_from_text(t):
+                yield t
 
-#     @staticmethod
-#     def extract_from_dict(
-#         data: Dict,
-#         out_datasets: List[tuple[str, SqlWrapper]],
-#         out_jscodes: List[tuple[str, str]],
-#     ):
-#         stack = [(data, "")]
-
-#         while len(stack) > 0:
-#             target, path = stack.pop()
-
-#             if isinstance(target, dict):
-#                 inputs = ((value, f"{path}.{key}") for key, value in target.items())
-#                 stack.extend(inputs)
-#                 continue
-
-#             if isinstance(target, list):
-#                 inputs = ((value, f"{path}[{idx}]") for idx, value in enumerate(target))
-#                 stack.extend(inputs)
-#                 continue
-
-#             if isinstance(target, SqlWrapper):
-#                 out_datasets.append((path[1:], target))
-
-#             if (
-#                 isinstance(target, str)
-#                 and target[: len(SqlExtractor.js_code_flag)]
-#                 == SqlExtractor.js_code_flag
-#                 and target[-len(SqlExtractor.js_code_flag) :]
-#                 == SqlExtractor.js_code_flag
-#             ):
-#                 target = target[
-#                     len(SqlExtractor.js_code_flag)  : -len(SqlExtractor.js_code_flag)
-#                 ]
-#                 out_jscodes.append((path[1:], target))
+        else:
+            yield t
