@@ -87,6 +87,9 @@ class ResourceManager:
         self.element_cps_css = empty_resource()
 
         self.mermaid_cps_js = empty_resource()
+        self.mermaid_cps_css = empty_resource()
+
+        self.experimental_cps_js = empty_resource()
 
         self.zip_js = None
 
@@ -111,11 +114,26 @@ class ResourceManager:
         )
         return self
 
+    def register_experimental_cps(self):
+        self.experimental_cps_js = js_offline_resource(
+            _get_file_path_from_static("experimentalCps.iife.js"),
+            id="experimentalCps-js",
+        )
+        # self.experimental_cps_css = css_offline_resource(
+        #     _get_file_path_from_static("experimentalCps-style.css"),
+        #     id="experimentalCps-style",
+        # )
+        return self
+
     def register_mermaid_cps(self):
         self.mermaid_cps_js = js_offline_resource(
             _get_file_path_from_static("mermaidCps.iife.js"), id="mermaidCps-js"
         )
 
+        self.mermaid_cps_css = css_offline_resource(
+            _get_file_path_from_static("mermaidCps-style.css"), id="mermaidCps-style"
+        )
+
         return self
 
     def build_html(self, appConfig: str, svg_infos: List[Dict]) -> str:
