@@ -42,7 +42,7 @@ class PieChart(BaseChart):
     def _create_default_click_filter(self):
         self.click_filter("name", self.data, self.name)
 
-    def get_options_infos(self):
+    def _create_options_ex(self):
         opts = super().get_options()
 
         base_opt = {
@@ -78,5 +78,4 @@ class PieChart(BaseChart):
 
         opts["tooltip"] = {"trigger": "item"}
 
-        self._update_props_by_path(base_opt)
         return base_opt, self._updateInfos, []
